/*
 * Copyright 2005-2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.springframework.ws.server.endpoint.mapping;

import java.net.URI;

import org.springframework.ws.MockWebServiceMessageFactory;
import org.springframework.ws.context.DefaultMessageContext;
import org.springframework.ws.context.MessageContext;
import org.springframework.ws.transport.WebServiceConnection;
import org.springframework.ws.transport.context.DefaultTransportContext;
import org.springframework.ws.transport.context.TransportContextHolder;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import static org.easymock.EasyMock.*;

public class UriEndpointMappingTest {

	private UriEndpointMapping mapping;

	private MessageContext context;

	@Before
	public void setUp() throws Exception {
		mapping = new UriEndpointMapping();
		context = new DefaultMessageContext(new MockWebServiceMessageFactory());
	}

	@After
	public void clearContext() {
		TransportContextHolder.setTransportContext(null);
	}

	@Test
	public void getLookupKeyForMessage() throws Exception {
		WebServiceConnection connectionMock = createMock(WebServiceConnection.class);
		TransportContextHolder.setTransportContext(new DefaultTransportContext(connectionMock));

		URI uri = new URI("jms://exampleQueue");
		expect(connectionMock.getUri()).andReturn(uri);

		replay(connectionMock);

		Assert.assertEquals("Invalid lookup key", uri.toString(), mapping.getLookupKeyForMessage(context));

		verify(connectionMock);
	}

	@Test
	public void getLookupKeyForMessagePath() throws Exception {
		mapping.setUsePath(true);

		WebServiceConnection connectionMock = createMock(WebServiceConnection.class);
		TransportContextHolder.setTransportContext(new DefaultTransportContext(connectionMock));

		URI uri = new URI("http://example.com/foo/bar");
		expect(connectionMock.getUri()).andReturn(uri);

		replay(connectionMock);

		Assert.assertEquals("Invalid lookup key", "/foo/bar", mapping.getLookupKeyForMessage(context));

		verify(connectionMock);
	}

	@Test
	public void testValidateLookupKey() throws Exception {
		Assert.assertTrue("URI not valid", mapping.validateLookupKey("http://example.com/services"));
		Assert.assertFalse("URI not valid", mapping.validateLookupKey("some string"));
	}
}